#!/usr/bin/perl -w
use strict;

use cosmic;

sub EarlyExample {
    my $txt = shift;
    return ShowLine("(intro-in-unary $txt)");
}

sub ShowUnaryLesson {
  my $txt = "";

  $txt .= "# MATH introduce numbers (in unary notation)\n";
  $txt .= "# Here we count up, then count down, then go through primes, etc.\n";
  $txt .= "# There is a bunch of syntax around the numbers, but that\n";
  $txt .= "# can just be treated as noise at this point - it doesn't matter.\n";
  $txt .= "# Any 'words' written here are converted to arbitrary integers\n";
  $txt .= "# in the actual message.  Any word ending in -in-unary will be given\n";
  $txt .= "# in unary rather than the binary code used in the main body\n";
  $txt .= "# of the message - this is simpler to start with.\n";
  for (my $i=1; $i<=27; $i++)
    {
	$txt .= EarlyExample(ShowUnary($i));
    }
  
  for (my $i=26; $i>=1; $i--)
    {
	$txt .= EarlyExample(ShowUnary($i));
    }

  foreach my $i (2, 3, 5, 7, 11, 13, 17, 19, 23)
    {
	$txt .= EarlyExample(ShowUnary($i));
    }

  foreach my $i (1, 4, 9, 16, 25)
    {
	$txt .= EarlyExample(ShowUnary($i));
    }

  foreach my $i (1, 8, 27)
    {
	$txt .= EarlyExample(ShowUnary($i));
    }


  $txt .= "# MATH introduce equality for unary numbers\n";
  $txt .= "# The intro operator does nothing essential, and could be\n";
  $txt .= "# omitted - it just tags the first use of a new operator.\n";
  $txt .= "# The = operator is introduced alongside a duplication of\n";
  $txt .= "# unary numbers.  The meaning will not quite by nailed down\n";
  $txt .= "# until we see other relational operators.\n";

#  $txt .= ShowLine(Op("intro","="));
  my @examples = (1, 2, 3, 4, 5, 6, 7, 8, 1, 6, 2);
#prand(10,5);
  for (my $i=0; $i<=$#examples; $i++)
    {
      my $r = $examples[$i];
      $txt .= ShowLine(Op2("=-in-unary",ShowUnary($r),ShowUnary($r)));
    }

  $txt .= "# MATH now introduce other relational operators\n";
  $txt .= "# After this lesson, it should be clear what contexts\n";
  $txt .= "# < > and = are appropriate in.\n";

#  $txt .= ShowLine(Op("intro",">"));
  for (my $i=0; $i<=10; $i++)
    {
      my $r = irand(6);
      my $r2 = irand($r);
      $txt .= ShowLine(Op2(">-in-unary",ShowUnary($r+1),ShowUnary($r2)));
    }
#  $txt .= ShowLine(Op("intro","<"));
  for (my $i=0; $i<=10; $i++)
    {
      my $r = irand(6);
      my $r2 = irand($r);
      $txt .= ShowLine(Op2("<-in-unary",ShowUnary($r2),ShowUnary($r+1)));
    }
  $txt .= "# drive the lesson home\n";
  for (my $i=0; $i<=2; $i++) {
      for (my $j=0; $j<=2; $j++) {
	  my $r = $i;
	  my $r2 = $j;
	  my $cmp = "=-in-unary";
	  if ($r>$r2)
	  {
	      $cmp = ">-in-unary";
	  }
	  elsif ($r<$r2)
	  {
	      $cmp = "<-in-unary";
	  }
	  $txt .= ShowLine(Op2($cmp,ShowUnary($r),ShowUnary($r2)));
      }
  }
  $txt .= "# switch to binary labelling\n";
  for (my $i=0; $i<=2; $i++) {
      for (my $j=0; $j<=2; $j++) {
	  my $r = $i;
	  my $r2 = $j;
	  my $cmp = "=";
	  if ($r>$r2)
	  {
	      $cmp = ">";
	  }
	  elsif ($r<$r2)
	  {
	      $cmp = "<";
	  }
	  $txt .= ShowLine(Op2($cmp,ShowUnary($r),ShowUnary($r2)));
      }
  }
  $txt .= "# a few more random examples\n";
  for (my $i=0; $i<=10; $i++)
    {
      my $r = irand(6);
      my $r2 = irand(6);
      my $cmp = "=";
      if ($r>$r2)
	{
	  $cmp = ">";
	}
      elsif ($r<$r2)
	{
	  $cmp = "<";
	}
      $txt .= ShowLine(Op2($cmp,ShowUnary($r),ShowUnary($r2)));
    }

  return $txt;
};


ShowLesson(ShowUnaryLesson());

