
;   Author: Paul Fitzpatrick, paulfitz@csail.mit.edu
;   Copyright (c) 2003 Paul Fitzpatrick
;
;   This file is part of CosmicOS.
;
;   CosmicOS is free software; you can redistribute it and/or modify
;   it under the terms of the GNU General Public License as published by
;   the Free Software Foundation; either version 2 of the License, or
;   (at your option) any later version.
;
;   CosmicOS is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;   GNU General Public License for more details.
;
;   You should have received a copy of the GNU General Public License
;   along with CosmicOS; if not, write to the Free Software
;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


(define fritz-break-list
  (lambda (in)
    (if (> (length in) 0)
	(let ((top (car in))
	      (rem (cdr in)))
	  (if (list? top)
	      (cons (fritz-break-list top) (fritz-break-list rem))
	      (if (= top -1)
		  (list (fritz-break-list rem))
		  (cons top (fritz-break-list rem)))))
	'())))

(define fritzify-list
  (lambda (exp)
    (if (number? exp)
	exp
	(cons (length exp)
	      (if (> (length exp) 1)
		  (cons (fritzify-list (car exp)) 
			(cdr (fritzify-list (cdr exp))))
		  (if (> (length exp) 0)
		      (fritzify-list (car exp))
		      0))))))

(define defritzify-list
  (lambda (exp)
    (if (number? exp)
	exp
	(fritz-break-list
	 (let ((n (car exp)))
	   (if (> n 1)
	       (append (list (defritzify-list (car (cdr exp))) )
		       (defritzify-list (cons (- n 1) (cdr (cdr exp)))))
	       (if (> n 0)
		   (list (defritzify-list (cdr exp)))
		   '())))))))

(define tish
  (lambda (x)
    (> x 0)))

(define nish
  (lambda (x)
    (if x 1 0)))



(define fritz-1
  (lambda (x) (nish #t)))

(define fritz-2
  (lambda (x) (lambda (y) 
		(nish
		 (if (and (procedure? x) (procedure? y))
		     ((fritz-5 (x 102)) (y 102))
		     (equal? x y))))))

(define fritz-3
  (lambda (x) (lambda (y) (nish (> x y)))))

(define fritz-4
  (lambda (x) (lambda (y) (nish (< x y)))))

(define fritz-5
  (lambda (x) (lambda (y) (+ x y))))

(define fritz-6
  (lambda (x) (lambda (y) (* x y))))

(define fritz-7
  (lambda (x) (lambda (y) (- x y))))

(define fritz-8
  (lambda (x) (nish (not (tish x)))))

(define fritz-9
  (lambda (x) (lambda (y) (nish (<= x y)))))

(define fritz-10
  (lambda (x) (lambda (y) (nish (>= x y)))))

;; "assign" gets intercepted, value here doesn't matter
(define fritz-11
  (nish #f))

(define fritz-13
  (lambda (x) (lambda (y) (nish (and (tish x) (tish y))))))

(define fritz-14
  (lambda (x) (lambda (y) (nish (or (tish x) (tish y))))))

(define fritz-15
  (nish #f))

(define fritz-16
  (nish #t))

;; "define" gets intercepted, value here doesn't matter
(define fritz-17
  (nish #f))

;; "if" gets intercepted, value here doesn't matter
(define fritz-18
  (nish #f))

(define fritz-19
  (lambda (x)
    (nish (and (tish (x -5)) (tish (x 10)) (tish (x 15)) (tish (x 18)))))) ;; try a few samples - not real code

(define fritz-20
  (lambda (x)
    (nish
     (let loop ((n -10))   ;; try a few samples - not real code
       (if (tish (x n))
	   #t
	   (if (< n 20)
	       (loop (+ n 1))
	       #f))))))

(define fritz-21
  (lambda (x) (lambda (y) (cons x y))))

(define fritz-22
  (lambda (x) (car x)))

(define fritz-23
  (lambda (x) (cdr x)))

(define fritz-24
  (lambda (x) (nish (number? x))))


;;; fritz-25 is translator, see below

;;; "lambda" gets created in message, value here doesn't matter
(define fritz-26
  (nish #f))

(define fritz-27
  (lambda (x)
    (make-cell x)))

(define fritz-28
  (lambda (x)
    (lambda (y)
      (begin (set-cell-contents! x y)
	     (nish #t)))))

(define fritz-29
  (lambda (x)
    (cell-contents x)))

(define fritz-30
  (lambda (x)
    (fritzify-list
     (let loop ((n -50))   ;; try a few samples - not real code
       (let ((rest (if (< n 50)
		       (loop (+ n 1))
		       '())))
	 (if (tish (x n))
	     (append (list n) rest)
	     rest))))))

(define fritz-31
  (fritz-30 (lambda (x) (nish (>= x 0)))))

(define fritz-32
  #f)

(define fritz-34
  (lambda (x) (lambda (y) (quotient x y))))

(define fritz-35
  (fritzify-list (fritz-break-list fritz-primer)))




;; this is the core "translate" procedure.
;; takes SCHEME-style lists rather than FRITZ-style lists.
(define fritz-translate
  (lambda (exp0)
    (let ((exp (fritzify-list exp0)))
      (fritz-25 exp))))

;; this is the core "translate" procedure.
;; It converts the FRITZ expression it receives into SCHEME code.
;; takes FRITZ-style lists rather than SCHEME-style lists.
(define fritz-25
  (lambda (exp0)
    (let ((exp (defritzify-list exp0)))
      (if (not (number? exp))
	  (let ((cmd (car exp))
		(rem (cdr exp)))
	    (cond ((equal? cmd 0)
		   ;; expression evaluates to a procedure
		   (let ((formal (car rem))
			 (body (car (cdr rem))))
		     (list 'lambda
			   (list (intern 
				  (string-append "fritz-" 
						 (number->string formal))))
			   (fritz-translate body))))
		   ;; expression defines a procedure
		  ((equal? cmd 17)
		   (let ((formal (car rem))
			 (body (car (cdr rem))))
		     (list 'begin
			   (list 'define 
				 (intern 
				  (string-append "fritz-" 
						 (number->string formal)))
				 (fritz-translate body))
			   (nish #t))))
		  ;; assignment shorthand
		  ((equal? cmd 11)
		   (let ((formal (car rem))
			 (value (car (cdr rem)))
			 (body  (car (cdr (cdr rem)))))
		     (list
		      (list 'lambda
			    (list (intern 
				   (string-append "fritz-" 
						  (number->string formal))))
			    (fritz-translate body))
		      (fritz-translate value))))
		  ;; expression is an if statement
		  ((equal? cmd 18)
		   (let ((cnd (car rem))
			 (on1 (car (cdr rem)))
			 (on0 (car (cdr (cdr rem)))))
		     (list 'if
			   (list 'tish (fritz-translate cnd))
			   (fritz-translate on1)
			   (fritz-translate on0))))
		  ;; expression is a procedure call
		  (else (let ((func-id (fritz-translate cmd)))
			  (let ((func (if (number? func-id)
					  (intern 
					   (string-append "fritz-" 
							  (number->string func-id)))
					  func-id)))
			    (if (> (length rem) 0)
				(let loop ((n (length rem))
					   (arg rem)
					   (base func))
				  (if (> n 1)
				      (loop (- n 1) (cdr arg)
					    (list base (fritz-translate (car arg))))
				      (list base (fritz-translate (car arg)))))
				func))))))
	  exp))))



;; Execute a single FRITZ expression
;; Result must be true (integer 1)
(define fritz-translate-show
  (lambda (exp)
    (begin 
      (display "    Expression: ")
      (display exp)
      (display "\n")
      (let ((result (fritz-25 (fritzify-list exp))))
	(begin
	  (display "    Translation: ")
	  (display result)
	  (display "\n")
	  (let ((val (eval result user-initial-environment)))
	    (if (equal? val 1)
		(display "ok")
		(begin
		  (display "UNEXPECTED RESULT ")
		  (display val)))
	    (display "\n")))))))


